struct VERTEX_IN
{
 float4 position :POSITION;
 float4 texCoord :TEXCOORD0;
 float4 texCoord2 :TEXCOORD1;
 float4 normal   :NORMAL;
};

struct VERTEX_OUT
{
	float4 hPos	: POSITION;
	float3 N 	: TEXCOORD0;
	float3 L      	: TEXCOORD1;
	float3 V      	: TEXCOORD2;
	float2 texCoord : TEXCOORD3;	
	float2 texCoord2 : TEXCOORD4;
};

VERTEX_OUT vsStatic(VERTEX_IN In,
		uniform float4x4 worldViewProj,
		uniform float4x4 worldView,
		uniform float4x4 worldViewIT,
		uniform float4 lightPos)
{
 VERTEX_OUT Out = (VERTEX_OUT) 0;

 Out.hPos = mul(worldViewProj, In.position);
 Out.texCoord = In.texCoord.xy;
 Out.texCoord2 = In.texCoord2.xy;
 Out.V = -mul(worldView, In.position).xyz;
 Out.L = lightPos.xyz + lightPos.w * Out.V;
 Out.N = mul(worldViewIT, In.normal).xyz;
 
 return Out;
}

VERTEX_OUT vsDynamic(VERTEX_IN In,
		uniform float4x4 worldViewProj,
		uniform float4x4 worldView,
		uniform float4x4 worldViewIT,
		uniform float4x4 world,
		uniform float4 lightPos)
{
 VERTEX_OUT Out = (VERTEX_OUT) 0;

 Out.hPos = mul(worldViewProj, In.position);
 Out.texCoord = In.texCoord.xy;
 float2 WPos = mul(world, In.position).xz;
 WPos += float2(170, 255);
 WPos /= float2(340, 425);
 Out.texCoord2 = WPos;
 Out.V = -mul(worldView, In.position).xyz;
 Out.L = lightPos.xyz + lightPos.w * Out.V;
 Out.N = mul(worldViewIT, In.normal).xyz;

 return Out;
}

float4 ps(VERTEX_OUT In,
	  uniform sampler2D colorTex : register(s0),
	  uniform sampler2D lightMap : register(s1),
	  uniform sampler2D normalMap : register(s2),
      	  uniform float bumpHeight,
    	  uniform float4 amientLightColor,
	  uniform float4 diffuseLightColor,
	  uniform float4 specularLightColor,
	  uniform float4 lightAttenuation,
	  uniform float4 ambientColor,
	  uniform float4 diffuseColor,
	  uniform float4 specularColor,
	  uniform float4 emissiveColor,
	  uniform float3 lightDirection,
	  uniform float4 lightSpotParams):COLOR0
{
	float3 N = normalize(In.N);
	float d = length(In.L);
	float3 L = In.L / d;
	float3 V = normalize(In.V);
	
	float3 tN = tex2D(normalMap, In.texCoord.xy).rgb;
	tN.xy = tN.xy * 2.0 - 1.0;
	tN.z /= bumpHeight;
	tN = normalize(tN);
 
	float facing = tN.z;
	float4 lightmap = tex2D(lightMap, In.texCoord2) * facing  * 0.2;

	float4 texColor = tex2D(colorTex, In.texCoord);

	float attenuation = 1.0 / 
			(lightAttenuation.y +
			 lightAttenuation.z * d +
			 lightAttenuation.w * d * d);
	diffuseLightColor *= attenuation; 
	specularLightColor *= attenuation; 
	
	float4 ambient = amientLightColor * ambientColor;
	float4 diffuse = max(0,dot(N,L)) * diffuseColor * diffuseLightColor;
	float3 H = normalize(L + V);
	float4 specular = pow(max(0, dot(N,H)), specularColor.w)
		* float4(specularColor.xyz, 1) * specularLightColor;

	return texColor * (ambient + lightmap + diffuse) + specular + emissiveColor;
}